@@ -47,6 +47,18 @@ struct NautilusDesktopWindowDetails {
 G_DEFINE_TYPE (NautilusDesktopWindow, nautilus_desktop_window, 
 	       NAUTILUS_TYPE_WINDOW);
 
+static void
+nautilus_desktop_window_dispose (GObject *obj)
+{
+	NautilusDesktopWindow *window = NAUTILUS_DESKTOP_WINDOW (obj);
+
+	g_signal_handlers_disconnect_by_func (nautilus_preferences,
+					      nautilus_desktop_window_update_directory,
+					      window);
+
+	G_OBJECT_CLASS (nautilus_desktop_window_parent_class)->dispose (obj);
+}
+
 static void
 nautilus_desktop_window_constructed (GObject *obj)
 {
@@ -107,9 +119,10 @@ void
 nautilus_desktop_window_update_directory (NautilusDesktopWindow *window)
 {
 	GFile *location;
-	
+
 	g_assert (NAUTILUS_IS_DESKTOP_WINDOW (window));
-	
+
+	window->details->loaded = FALSE;
 	location = g_file_new_for_uri (EEL_DESKTOP_URI);
 	nautilus_window_go_to (NAUTILUS_WINDOW (window), location);
 	window->details->loaded = TRUE;
@@ -133,8 +146,7 @@ nautilus_desktop_window_screen_size_changed (GdkScreen             *screen,
 }
 
 NautilusDesktopWindow *
-nautilus_desktop_window_new (NautilusApplication *application,
-			     GdkScreen           *screen)
+nautilus_desktop_window_new (GdkScreen *screen)
 {
 	NautilusDesktopWindow *window;
 	int width_request, height_request;
@@ -144,7 +156,6 @@ nautilus_desktop_window_new (NautilusApplication *application,
 
 	window = g_object_new (NAUTILUS_TYPE_DESKTOP_WINDOW,
 			       "disable-chrome", TRUE,
-			       "app", application,
 			       "width_request", width_request,
 			       "height_request", height_request,
 			       "screen", screen,
@@ -278,6 +289,7 @@ nautilus_desktop_window_class_init (NautilusDesktopWindowClass *klass)
 	GObjectClass *oclass = G_OBJECT_CLASS (klass);
 
 	oclass->constructed = nautilus_desktop_window_constructed;
+	oclass->dispose = nautilus_desktop_window_dispose;
 
 	wclass->realize = realize;
 	wclass->unrealize = unrealize;
